<template>
  <byjove-products></byjove-products>
</template>

<script>
import {ByjoveProducts} from 'byjove'

export default {
    name: 'Products',
    components: {
        ByjoveProducts,
    },
}
</script>
